module Intra
  class Session
    def initialize(env)
      @env = env
      @current_user = nil
      @cookies = nil
      check_timeout
    end

    def add_cookie_to_headers(headers)
      if current_user&.remember_token
        value = cookie_value
        value['remember_token'] = current_user.remember_token
        options = cookie_options.merge value: value.to_json
        Rack::Utils.set_cookie_header! headers, INTRA_COOKIE, options
      end
    end

    def current_user
      if remember_token.present?
        @current_user ||= user_model.find_by(remember_token: remember_token)
      end
      @current_user
    end

    def sign_in(user)
      @current_user = user
    end

    def sign_out
      current_user.reset_remember_token! if signed_in?

      @current_user = nil
      cookies.delete INTRA_COOKIE
    end

    def signed_in?
      current_user.present?
    end

    def signed_out?
      !signed_in?
    end

    def check_timeout
      sign_out if timed_out?
    end

    private

    def timed_out?
      return false unless Intra.timeout_in.present?
      return false unless last_activity_at.present?

      Time.current - Intra.timeout_in > last_activity_at
    end

    def cookie
      @cookie ||= JSON.parse cookies[INTRA_COOKIE] rescue {}
    end

    def remember_token
      cookie['remember_token']
    end

    def last_activity_at
      cookie['last_activity_at']
    end

    def cookies
      @cookies ||= ActionDispatch::Request.new(@env).cookie_jar
    end

    def user_model
      Intra.user_class.constantize
    end

    def cookie_options
      {
        expires: Intra.remember_for.from_now,
        path: '/',
        secure: Rails.env.production?,
        value: cookie_value.to_json
      }
    end

    def cookie_value
      {
        remember_token: remember_token,
        last_activity_at: Time.current
      }
    end

    INTRA_COOKIE = '_intra_cookie'.freeze
  end
end
